@@ -2,11 +2,7 @@
 
 /**
  * This file is part of the Nette Framework (http://nette.org)
- *
  * Copyright (c) 2004 David Grudl (http://davidgrudl.com)
- *
- * For the full copyright and license information, please view
- * the file license.txt that was distributed with this source code.
  */
 
 namespace Nette\Application\Responses;
@@ -36,7 +32,7 @@ class ForwardResponse extends Nette\Object implements Nette\Application\IRespons
 	/**
 	 * @return Nette\Application\Request
 	 */
-	final public function getRequest()
+	public function getRequest()
 	{
 		return $this->request;
 	}
