@@ -2,11 +2,7 @@
 
 /**
  * This file is part of the Nette Framework (http://nette.org)
- *
  * Copyright (c) 2004 David Grudl (http://davidgrudl.com)
- *
- * For the full copyright and license information, please view
- * the file license.txt that was distributed with this source code.
  */
 
 namespace Nette\Application\Responses;
@@ -46,7 +42,7 @@ class RedirectResponse extends Nette\Object implements Nette\Application\IRespon
 	/**
 	 * @return string
 	 */
-	final public function getUrl()
+	public function getUrl()
 	{
 		return $this->url;
 	}
@@ -55,7 +51,7 @@ class RedirectResponse extends Nette\Object implements Nette\Application\IRespon
 	/**
 	 * @return int
 	 */
-	final public function getCode()
+	public function getCode()
 	{
 		return $this->code;
 	}
