@@ -2,11 +2,7 @@
 
 /**
  * This file is part of the Nette Framework (http://nette.org)
- *
  * Copyright (c) 2004 David Grudl (http://davidgrudl.com)
- *
- * For the full copyright and license information, please view
- * the file license.txt that was distributed with this source code.
  */
 
 namespace Nette\Application\Responses;
@@ -39,7 +35,7 @@ class TextResponse extends Nette\Object implements Nette\Application\IResponse
 	/**
 	 * @return mixed
 	 */
-	final public function getSource()
+	public function getSource()
 	{
 		return $this->source;
 	}
