@@ -2,11 +2,7 @@
 
 /**
  * This file is part of the Nette Framework (http://nette.org)
- *
  * Copyright (c) 2004 David Grudl (http://davidgrudl.com)
- *
- * For the full copyright and license information, please view
- * the file license.txt that was distributed with this source code.
  */
 
 namespace Nette\Application\Routers;
@@ -148,9 +144,16 @@ class Route extends Nette\Object implements Application\IRouter
 
 		// 1) URL MASK
 		$url = $httpRequest->getUrl();
+		$re = $this->re;
 
 		if ($this->type === self::HOST) {
 			$path = '//' . $url->getHost() . $url->getPath();
+			$host = array_reverse(explode('.', $url->getHost()));
+			$re = strtr($re, array(
+				'/%basePath%/' => preg_quote($url->getBasePath(), '#'),
+				'%tld%' => $host[0],
+				'%domain%' => isset($host[1]) ? "$host[1]\\.$host[0]" : $host[0],
+			));
 
 		} elseif ($this->type === self::RELATIVE) {
 			$basePath = $url->getBasePath();
@@ -167,7 +170,7 @@ class Route extends Nette\Object implements Application\IRouter
 			$path = rtrim($path, '/') . '/';
 		}
 
-		if (!$matches = Strings::match($path, $this->re)) {
+		if (!$matches = Strings::match($path, $re)) {
 			// stop, not matched
 			return NULL;
 		}
@@ -223,6 +226,12 @@ class Route extends Nette\Object implements Application\IRouter
 			}
 		}
 
+		if (isset($this->metadata[NULL][self::FILTER_IN])) {
+			$params = call_user_func($this->metadata[NULL][self::FILTER_IN], $params);
+			if ($params === NULL) {
+				return NULL;
+			}
+		}
 
 		// 5) BUILD Request
 		if (!isset($params[self::PRESENTER_KEY])) {
@@ -269,6 +278,13 @@ class Route extends Nette\Object implements Application\IRouter
 		$presenter = $appRequest->getPresenterName();
 		$params[self::PRESENTER_KEY] = $presenter;
 
+		if (isset($metadata[NULL][self::FILTER_OUT])) {
+			$params = call_user_func($metadata[NULL][self::FILTER_OUT], $params);
+			if ($params === NULL) {
+				return NULL;
+			}
+		}
+
 		if (isset($metadata[self::MODULE_KEY])) { // try split into module and [submodule:]presenter parts
 			$module = $metadata[self::MODULE_KEY];
 			if (isset($module['fixity']) && strncasecmp($presenter, $module[self::VALUE] . ':', strlen($module[self::VALUE]) + 1) === 0) {
@@ -368,6 +384,28 @@ class Route extends Nette\Object implements Application\IRouter
 		} while (TRUE);
 
 
+		// absolutize path
+		if ($this->type === self::RELATIVE) {
+			$url = '//' . $refUrl->getAuthority() . $refUrl->getBasePath() . $url;
+
+		} elseif ($this->type === self::PATH) {
+			$url = '//' . $refUrl->getAuthority() . $url;
+
+		} else {
+			$host = array_reverse(explode('.', $refUrl->getHost()));
+			$url = strtr($url, array(
+				'/%basePath%/' => $refUrl->getBasePath(),
+				'%tld%' => $host[0],
+				'%domain%' => isset($host[1]) ? "$host[1].$host[0]" : $host[0],
+			));
+		}
+
+		if (strpos($url, '//', 2) !== FALSE) {
+			return NULL; // TODO: implement counterpart in match() ?
+		}
+
+		$url = ($this->flags & self::SECURED ? 'https:' : 'http:') . $url;
+
 		// build query string
 		if ($this->xlat) {
 			$params = self::renameKeys($params, $this->xlat);
@@ -379,20 +417,6 @@ class Route extends Nette\Object implements Application\IRouter
 			$url .= '?' . $query;
 		}
 
-		// absolutize path
-		if ($this->type === self::RELATIVE) {
-			$url = '//' . $refUrl->getAuthority() . $refUrl->getBasePath() . $url;
-
-		} elseif ($this->type === self::PATH) {
-			$url = '//' . $refUrl->getAuthority() . $url;
-		}
-
-		if (strpos($url, '//', 2) !== FALSE) {
-			return NULL; // TODO: implement counterpart in match() ?
-		}
-
-		$url = ($this->flags & self::SECURED ? 'https:' : 'http:') . $url;
-
 		return $url;
 	}
 
@@ -506,8 +530,9 @@ class Route extends Nette\Object implements Application\IRouter
 			array_unshift($sequence, $name);
 
 			if ($name[0] === '?') { // "foo" parameter
-				$re = '(?:' . preg_quote(substr($name, 1), '#') . '|' . $pattern . ')' . $re;
-				$sequence[1] = substr($name, 1) . $sequence[1];
+				$name = substr($name, 1);
+				$re = $pattern ? '(?:' . preg_quote($name, '#') . "|$pattern)$re" : preg_quote($name, '#') . $re;
+				$sequence[1] = $name . $sequence[1];
 				continue;
 			}
 
@@ -630,6 +655,7 @@ class Route extends Nette\Object implements Application\IRouter
 
 	/**
 	 * Proprietary cache aim.
+	 * @internal
 	 * @return string|FALSE
 	 */
 	public function getTargetPresenter()
