@@ -2,11 +2,7 @@
 
 /**
  * This file is part of the Nette Framework (http://nette.org)
- *
  * Copyright (c) 2004 David Grudl (http://davidgrudl.com)
- *
- * For the full copyright and license information, please view
- * the file license.txt that was distributed with this source code.
  */
 
 namespace Nette\Application\UI;
@@ -21,20 +17,20 @@ use Nette;
  */
 class Multiplier extends PresenterComponent
 {
-	/** @var Nette\Callback */
+	/** @var callable */
 	private $factory;
 
 
 	public function __construct($factory)
 	{
 		parent::__construct();
-		$this->factory = new Nette\Callback($factory);
+		$this->factory = Nette\Utils\Callback::check($factory);
 	}
 
 
 	protected function createComponent($name)
 	{
-		return $this->factory->invoke($name, $this);
+		return call_user_func($this->factory, $name, $this);
 	}
 
 }
