@@ -2,11 +2,7 @@
 
 /**
  * This file is part of the Nette Framework (http://nette.org)
- *
  * Copyright (c) 2004 David Grudl (http://davidgrudl.com)
- *
- * For the full copyright and license information, please view
- * the file license.txt that was distributed with this source code.
  */
 
 namespace Nette\Application;
@@ -28,7 +24,7 @@ class AbortException extends \Exception
  */
 class ApplicationException extends \Exception
 {
-	}
+}
 
 
 /**
@@ -50,13 +46,7 @@ class BadRequestException extends \Exception
 
 	public function __construct($message = '', $code = 0, \Exception $previous = NULL)
 	{
-		if ($code < 200 || $code > 504) {
-			$code = $this->defaultCode;
-		}
-
-		{
-			parent::__construct($message, $code, $previous);
-		}
+		parent::__construct($message, $code < 200 || $code > 504 ? $this->defaultCode : $code, $previous);
 	}
 
 }
