@@ -2,11 +2,7 @@
 
 /**
  * This file is part of the Nette Framework (http://nette.org)
- *
  * Copyright (c) 2004 David Grudl (http://davidgrudl.com)
- *
- * For the full copyright and license information, please view
- * the file license.txt that was distributed with this source code.
  */
 
 namespace Nette\Database;
@@ -18,16 +14,30 @@ use Nette;
  * SQL literal value.
  *
  * @author     Jakub Vrana
+ * @author     Jan Skrasek
  */
 class SqlLiteral extends Nette\Object
 {
 	/** @var string */
-	private $value = '';
+	private $value;
+
+	/** @var array */
+	private $parameters;
 
 
-	public function __construct($value)
+	public function __construct($value, array $parameters = array())
 	{
 		$this->value = (string) $value;
+		$this->parameters = $parameters;
+	}
+
+
+	/**
+	 * @return array
+	 */
+	public function getParameters()
+	{
+		return $this->parameters;
 	}
 
 
