@@ -2,18 +2,13 @@
 
 /**
  * This file is part of the Nette Framework (http://nette.org)
- *
  * Copyright (c) 2004 David Grudl (http://davidgrudl.com)
- *
- * For the full copyright and license information, please view
- * the file license.txt that was distributed with this source code.
  */
 
 namespace Nette\Database\Table;
 
 use Nette,
-	Nette\Database\ISupplementalDriver,
-	PDO;
+	Nette\Database\ISupplementalDriver;
 
 
 /**
@@ -25,11 +20,14 @@ use Nette,
  *
  * @property-read string $sql
  */
-class Selection extends Nette\Object implements \Iterator, \ArrayAccess, \Countable
+class Selection extends Nette\Object implements \Iterator, IRowContainer, \ArrayAccess, \Countable
 {
 	/** @var Nette\Database\Connection */
 	protected $connection;
 
+	/** @var Nette\Database\IReflection */
+	protected $reflection;
+
 	/** @var Nette\Caching\Cache */
 	protected $cache;
 
@@ -45,25 +43,28 @@ class Selection extends Nette\Object implements \Iterator, \ArrayAccess, \Counta
 	/** @var string|bool primary column sequence name, FALSE for autodetection */
 	protected $primarySequence = FALSE;
 
-	/** @var ActiveRow[] data read from database in [primary key => ActiveRow] format */
+	/** @var IRow[] data read from database in [primary key => IRow] format */
 	protected $rows;
 
-	/** @var ActiveRow[] modifiable data in [primary key => ActiveRow] format */
+	/** @var IRow[] modifiable data in [primary key => IRow] format */
 	protected $data;
 
 	/** @var bool */
 	protected $dataRefreshed = FALSE;
 
-	/** @var Selection[] */
-	protected $referenced = array();
+	/** @var mixed cache array of Selection and GroupedSelection prototypes */
+	protected $globalRefCache;
 
-	/** @var array of [sqlQuery-hash => grouped data]; used by GroupedSelection */
-	protected $referencing = array();
+	/** @var mixed */
+	protected $refCache;
 
-	/** @var GroupedSelection[] cached array of GroupedSelection prototypes */
-	protected $referencingPrototype = array();
+	/** @var string */
+	protected $generalCacheKey;
 
-	/** @var array of [conditions => [key => ActiveRow]]; used by GroupedSelection */
+	/** @var string */
+	protected $specificCacheKey;
+
+	/** @var array of [conditions => [key => IRow]]; used by GroupedSelection */
 	protected $aggregation = array();
 
 	/** @var array of touched columns */
@@ -75,26 +76,24 @@ class Selection extends Nette\Object implements \Iterator, \ArrayAccess, \Counta
 	/** @var bool should instance observe accessed columns caching */
 	protected $observeCache = FALSE;
 
-	/** @var bool recheck referencing keys */
-	protected $checkReferenced = FALSE;
-
 	/** @var array of primary key values */
 	protected $keys = array();
 
 
 	/**
 	 * Creates filtered table representation.
-	 * @param  string  database table name
 	 * @param  Nette\Database\Connection
+	 * @param  string  database table name
 	 */
-	public function __construct($table, Nette\Database\Connection $connection)
+	public function __construct(Nette\Database\Connection $connection, $table, Nette\Database\IReflection $reflection, Nette\Caching\IStorage $cacheStorage = NULL)
 	{
 		$this->name = $table;
 		$this->connection = $connection;
-		$reflection = $connection->getDatabaseReflection();
+		$this->reflection = $reflection;
+		$this->cache = $cacheStorage ? new Nette\Caching\Cache($cacheStorage, 'Nette.Database.' . md5($connection->getDsn())) : NULL;
 		$this->primary = $reflection->getPrimary($table);
 		$this->sqlBuilder = new SqlBuilder($table, $connection, $reflection);
-		$this->cache = $connection->getCache();
+		$this->refCache = & $this->getRefTable($refPath)->globalRefCache[$refPath];
 	}
 
 
@@ -119,6 +118,15 @@ class Selection extends Nette\Object implements \Iterator, \ArrayAccess, \Counta
 	}
 
 
+	/**
+	 * @return Nette\Database\IReflection
+	 */
+	public function getDatabaseReflection()
+	{
+		return $this->reflection;
+	}
+
+
 	/**
 	 * @return string
 	 */
@@ -191,7 +199,10 @@ class Selection extends Nette\Object implements \Iterator, \ArrayAccess, \Counta
 	public function getPreviousAccessedColumns()
 	{
 		if ($this->cache && $this->previousAccessedColumns === NULL) {
-			$this->accessedColumns = $this->previousAccessedColumns = $this->cache->load($this->getCacheKey());
+			$this->accessedColumns = $this->previousAccessedColumns = $this->cache->load($this->getGeneralCacheKey());
+			if ($this->previousAccessedColumns === NULL) {
+				$this->previousAccessedColumns = array();
+			}
 		}
 
 		return array_keys(array_filter((array) $this->previousAccessedColumns));
@@ -214,7 +225,7 @@ class Selection extends Nette\Object implements \Iterator, \ArrayAccess, \Counta
 	/**
 	 * Returns row specified by primary key.
 	 * @param  mixed primary key
-	 * @return ActiveRow or FALSE if there is no such row
+	 * @return IRow or FALSE if there is no such row
 	 */
 	public function get($key)
 	{
@@ -224,8 +235,7 @@ class Selection extends Nette\Object implements \Iterator, \ArrayAccess, \Counta
 
 
 	/**
-	 * Returns next row of result.
-	 * @return ActiveRow or FALSE if there is no row
+	 * @inheritDoc
 	 */
 	public function fetch()
 	{
@@ -237,18 +247,20 @@ class Selection extends Nette\Object implements \Iterator, \ArrayAccess, \Counta
 
 
 	/**
-	 * Returns all rows as associative array.
-	 * @param  string
-	 * @param  string column name used for an array value or NULL for the whole row
-	 * @return array
+	 * @inheritDoc
 	 */
-	public function fetchPairs($key, $value = NULL)
+	public function fetchPairs($key = NULL, $value = NULL)
 	{
-		$return = array();
-		foreach ($this as $row) {
-			$return[is_object($row[$key]) ? (string) $row[$key] : $row[$key]] = ($value ? $row[$value] : $row);
-		}
-		return $return;
+		return Nette\Database\Helpers::toPairs(iterator_to_array($this), $key, $value);
+	}
+
+
+	/**
+	 * @inheritDoc
+	 */
+	public function fetchAll()
+	{
+		return iterator_to_array($this);
 	}
 
 
@@ -263,17 +275,17 @@ class Selection extends Nette\Object implements \Iterator, \ArrayAccess, \Counta
 	public function select($columns)
 	{
 		$this->emptyResultSet();
-		$this->sqlBuilder->addSelect($columns);
+		call_user_func_array(array($this->sqlBuilder, 'addSelect'), func_get_args());
 		return $this;
 	}
 
 
 	/**
-	 * Method is deprecated, use wherePrimary() instead.
-	 * @return self
+	 * @deprecated
 	 */
 	public function find($key)
 	{
+		trigger_error(__METHOD__ . '() is deprecated; use $selection->wherePrimary() instead.', E_USER_DEPRECATED);
 		return $this->wherePrimary($key);
 	}
 
@@ -285,14 +297,18 @@ class Selection extends Nette\Object implements \Iterator, \ArrayAccess, \Counta
 	 */
 	public function wherePrimary($key)
 	{
-		if (is_array($this->primary) && Nette\Utils\Validators::isList($key)) {
-			foreach ($this->primary as $i => $primary) {
-				$this->where($primary, $key[$i]);
+		if (is_array($this->primary) && Nette\Utils\Arrays::isList($key)) {
+			if (isset($key[0]) && is_array($key[0])) {
+				$this->where($this->primary, $key);
+			} else {
+				foreach ($this->primary as $i => $primary) {
+					$this->where($this->name . '.' . $primary, $key[$i]);
+				}
 			}
-		} elseif (is_array($key)) { // key contains column names
+		} elseif (is_array($key) && !Nette\Utils\Arrays::isList($key)) { // key contains column names
 			$this->where($key);
 		} else {
-			$this->where($this->getPrimary(), $key);
+			$this->where($this->name . '.' . $this->getPrimary(), $key);
 		}
 
 		return $this;
@@ -308,7 +324,7 @@ class Selection extends Nette\Object implements \Iterator, \ArrayAccess, \Counta
 	 */
 	public function where($condition, $parameters = array())
 	{
-		if (is_array($condition)) { // where(array('column1' => 1, 'column2 > ?' => 2))
+		if (is_array($condition) && $parameters === array()) { // where(array('column1' => 1, 'column2 > ?' => 2))
 			foreach ($condition as $key => $val) {
 				if (is_int($key)) {
 					$this->where($val); // where('full condition')
@@ -319,10 +335,8 @@ class Selection extends Nette\Object implements \Iterator, \ArrayAccess, \Counta
 			return $this;
 		}
 
-		if (call_user_func_array(array($this->sqlBuilder, 'addWhere'), func_get_args())) {
-			$this->emptyResultSet();
-		}
-
+		$this->emptyResultSet();
+		call_user_func_array(array($this->sqlBuilder, 'addWhere'), func_get_args());
 		return $this;
 	}
 
@@ -335,7 +349,7 @@ class Selection extends Nette\Object implements \Iterator, \ArrayAccess, \Counta
 	public function order($columns)
 	{
 		$this->emptyResultSet();
-		$this->sqlBuilder->addOrder($columns);
+		call_user_func_array(array($this->sqlBuilder, 'addOrder'), func_get_args());
 		return $this;
 	}
 
@@ -360,22 +374,43 @@ class Selection extends Nette\Object implements \Iterator, \ArrayAccess, \Counta
 	 * @param  int
 	 * @return self
 	 */
-	public function page($page, $itemsPerPage)
+	public function page($page, $itemsPerPage, & $numOfPages = NULL)
 	{
+		if (func_get_args() > 2) {
+			$numOfPages = (int) ceil($this->count('*') / $itemsPerPage);
+		}
 		return $this->limit($itemsPerPage, ($page - 1) * $itemsPerPage);
 	}
 
 
 	/**
-	 * Sets group clause, more calls rewrite old values.
-	 * @param  string
+	 * Sets group clause, more calls rewrite old value.
 	 * @param  string
 	 * @return self
 	 */
-	public function group($columns, $having = NULL)
+	public function group($columns)
 	{
 		$this->emptyResultSet();
-		$this->sqlBuilder->setGroup($columns, $having);
+		if (func_num_args() === 2 && strpos($columns, '?') === FALSE) {
+			trigger_error('Calling ' . __METHOD__ . '() with second argument is deprecated; use $selection->having() instead.', E_USER_DEPRECATED);
+			$this->having(func_get_arg(1));
+			$this->sqlBuilder->setGroup($columns);
+		} else {
+			call_user_func_array(array($this->sqlBuilder, 'setGroup'), func_get_args());
+		}
+		return $this;
+	}
+
+
+	/**
+	 * Sets having clause, more calls rewrite old value.
+	 * @param  string
+	 * @return self
+	 */
+	public function having($having)
+	{
+		$this->emptyResultSet();
+		call_user_func_array(array($this->sqlBuilder, 'setHaving'), func_get_args());
 		return $this;
 	}
 
@@ -458,6 +493,10 @@ class Selection extends Nette\Object implements \Iterator, \ArrayAccess, \Counta
 
 		$this->observeCache = $this;
 
+		if ($this->primary === NULL && $this->sqlBuilder->getSelect() === NULL) {
+			throw new Nette\InvalidStateException('Table with no primary key requires an explicit select clause.');
+		}
+
 		try {
 			$result = $this->query($this->getSql());
 
@@ -473,8 +512,7 @@ class Selection extends Nette\Object implements \Iterator, \ArrayAccess, \Counta
 
 		$this->rows = array();
 		$usedPrimary = TRUE;
-		$result->setFetchMode(PDO::FETCH_ASSOC);
-		foreach ($result as $key => $row) {
+		foreach ($result->getPdoStatement() as $key => $row) {
 			$row = $this->createRow($result->normalizeRow($row));
 			$primary = $row->getSignature(FALSE);
 			$usedPrimary = $usedPrimary && $primary;
@@ -496,9 +534,9 @@ class Selection extends Nette\Object implements \Iterator, \ArrayAccess, \Counta
 	}
 
 
-	protected function createSelectionInstance($table = NULL)
+	public function createSelectionInstance($table = NULL)
 	{
-		return new Selection($table ?: $this->name, $this->connection);
+		return new Selection($this->connection, $table ?: $this->name, $this->reflection, $this->cache ? $this->cache->getStorage() : NULL);
 	}
 
 
@@ -514,16 +552,29 @@ class Selection extends Nette\Object implements \Iterator, \ArrayAccess, \Counta
 	}
 
 
-	protected function emptyResultSet()
+	protected function emptyResultSet($saveCache = TRUE)
 	{
+		if ($this->rows !== NULL && $saveCache) {
+			$this->saveCacheState();
+		}
+
 		$this->rows = NULL;
+		$this->specificCacheKey = NULL;
+		$this->generalCacheKey = NULL;
+		$this->refCache['referencingPrototype'] = array();
 	}
 
 
 	protected function saveCacheState()
 	{
-		if ($this->observeCache === $this && $this->cache && !$this->sqlBuilder->getSelect() && $this->accessedColumns != $this->previousAccessedColumns) {
-			$this->cache->save($this->getCacheKey(), $this->accessedColumns);
+		if ($this->observeCache === $this && $this->cache && !$this->sqlBuilder->getSelect() && $this->accessedColumns !== $this->previousAccessedColumns) {
+			$previousAccessed = (array) $this->cache->load($this->getGeneralCacheKey());
+			$accessed = (array) $this->accessedColumns;
+			$needSave = array_intersect_key($accessed, $previousAccessed) !== $accessed;
+			if ($needSave) {
+				$this->cache->save($this->getGeneralCacheKey(), $previousAccessed + $accessed);
+				$this->previousAccessedColumns = NULL;
+			}
 		}
 	}
 
@@ -539,18 +590,46 @@ class Selection extends Nette\Object implements \Iterator, \ArrayAccess, \Counta
 
 
 	/**
-	 * Returns cache key for selected columns caching
+	 * Loads refCache references
+	 */
+	protected function loadRefCache()
+	{
+	}
+
+
+	/**
+	 * Returns general cache key indenpendent on query parameters or sql limit
+	 * Used e.g. for previously accessed columns caching
 	 * @return string
 	 */
-	protected function getCacheKey()
+	protected function getGeneralCacheKey()
 	{
-		return md5(serialize(array(__CLASS__, $this->name, $this->sqlBuilder->getConditions())));
+		if ($this->generalCacheKey) {
+			return $this->generalCacheKey;
+		}
+
+		return $this->generalCacheKey = md5(serialize(array(__CLASS__, $this->name, $this->sqlBuilder->getConditions())));
+	}
+
+
+	/**
+	 * Returns object specific cache key dependent on query parameters
+	 * Used e.g. for reference memory caching
+	 * @return string
+	 */
+	protected function getSpecificCacheKey()
+	{
+		if ($this->specificCacheKey) {
+			return $this->specificCacheKey;
+		}
+
+		return $this->specificCacheKey = md5($this->getSql() . json_encode($this->sqlBuilder->getParameters()));
 	}
 
 
 	/**
 	 * @internal
-	 * @param  string|NULL column name or (NULL to reload all columns & disable columns cache)
+	 * @param  string|NULL column name or NULL to reload all columns
 	 * @param  bool
 	 */
 	public function accessColumn($key, $selectColumn = TRUE)
@@ -567,8 +646,21 @@ class Selection extends Nette\Object implements \Iterator, \ArrayAccess, \Counta
 		}
 
 		if ($selectColumn && !$this->sqlBuilder->getSelect() && $this->previousAccessedColumns && ($key === NULL || !isset($this->previousAccessedColumns[$key]))) {
-			$this->previousAccessedColumns = FALSE;
-			$this->emptyResultSet();
+			if ($this->sqlBuilder->getLimit()) {
+				$generalCacheKey = $this->generalCacheKey;
+				$primaries = array();
+				foreach ((array) $this->rows as $row) {
+					$primary = $row->getPrimary();
+					$primaries[] = is_array($primary) ? array_values($primary) : $primary;
+				}
+			}
+			$this->previousAccessedColumns = array();
+			$this->emptyResultSet(FALSE);
+			if ($this->sqlBuilder->getLimit()) {
+				$this->sqlBuilder->setLimit(NULL, NULL);
+				$this->wherePrimary($primaries);
+				$this->generalCacheKey = $generalCacheKey;
+			}
 			$this->dataRefreshed = TRUE;
 
 			if ($key === NULL) {
@@ -609,32 +701,54 @@ class Selection extends Nette\Object implements \Iterator, \ArrayAccess, \Counta
 
 	/**
 	 * Inserts row in a table.
-	 * @param  mixed array($column => $value)|Traversable for single row insert or Selection|string for INSERT ... SELECT
-	 * @return ActiveRow or FALSE in case of an error or number of affected rows for INSERT ... SELECT
+	 * @param  array|\Traversable|Selection array($column => $value)|\Traversable|Selection for INSERT ... SELECT
+	 * @return IRow|int|bool Returns IRow or number of affected rows for Selection or table without primary key
 	 */
 	public function insert($data)
 	{
 		if ($data instanceof Selection) {
-			$data = $data->getSql();
+			$data = new Nette\Database\SqlLiteral($data->getSql(), $data->getSqlBuilder()->getParameters());
 
 		} elseif ($data instanceof \Traversable) {
 			$data = iterator_to_array($data);
 		}
 
 		$return = $this->connection->query($this->sqlBuilder->buildInsertQuery(), $data);
-		$this->checkReferenced = TRUE;
 
-		if (!is_array($data)) {
-			return $return->rowCount();
+		if ($data instanceof Nette\Database\SqlLiteral || $this->primary === NULL) {
+			return $return->getRowCount();
 		}
 
-		if (!is_array($this->primary) && !isset($data[$this->primary]) && ($id = $this->connection->lastInsertId($this->getPrimarySequence()))) {
-			$data[$this->primary] = $id;
+		$primaryKey = $this->connection->getInsertId($this->getPrimarySequence());
+		if (is_array($this->getPrimary())) {
+			$primaryKey = array();
+
+			foreach ((array) $this->getPrimary() as $key) {
+				if (!isset($data[$key])) {
+					return $data;
+				}
+
+				$primaryKey[$key] = $data[$key];
+			}
+			if (count($primaryKey) === 1) {
+				$primaryKey = reset($primaryKey);
+			}
 		}
 
-		$row = $this->createRow($data);
-		if ($signature = $row->getSignature(FALSE)) {
-			$this->rows[$signature] = $row;
+		$row = $this->createSelectionInstance()
+			->select('*')
+			->wherePrimary($primaryKey)
+			->fetch();
+
+		$this->loadRefCache();
+		if ($this->rows !== NULL) {
+			if ($signature = $row->getSignature(FALSE)) {
+				$this->rows[$signature] = $row;
+				$this->data[$signature] = $row;
+			} else {
+				$this->rows[] = $row;
+				$this->data[] = $row;
+			}
 		}
 
 		return $row;
@@ -645,7 +759,7 @@ class Selection extends Nette\Object implements \Iterator, \ArrayAccess, \Counta
 	 * Updates all rows in result set.
 	 * Joins in UPDATE are supported only in MySQL
 	 * @param  array|\Traversable ($column => $value)
-	 * @return int number of affected rows or FALSE in case of an error
+	 * @return int number of affected rows
 	 */
 	public function update($data)
 	{
@@ -663,17 +777,17 @@ class Selection extends Nette\Object implements \Iterator, \ArrayAccess, \Counta
 		return $this->connection->queryArgs(
 			$this->sqlBuilder->buildUpdateQuery(),
 			array_merge(array($data), $this->sqlBuilder->getParameters())
-		)->rowCount();
+		)->getRowCount();
 	}
 
 
 	/**
 	 * Deletes all rows in result set.
-	 * @return int number of affected rows or FALSE in case of an error
+	 * @return int number of affected rows
 	 */
 	public function delete()
 	{
-		return $this->query($this->sqlBuilder->buildDeleteQuery())->rowCount();
+		return $this->query($this->sqlBuilder->buildDeleteQuery())->getRowCount();
 	}
 
 
@@ -684,44 +798,35 @@ class Selection extends Nette\Object implements \Iterator, \ArrayAccess, \Counta
 	 * Returns referenced row.
 	 * @param  string
 	 * @param  string
-	 * @param  bool  checks if rows contains the same primary value relations
+	 * @param  mixed   primary key to check for $table and $column references
 	 * @return Selection or array() if the row does not exist
 	 */
-	public function getReferencedTable($table, $column, $checkReferenced = FALSE)
+	public function getReferencedTable($table, $column, $checkPrimaryKey)
 	{
-		$referenced = & $this->getRefTable($refPath)->referenced[$refPath . "$table.$column"];
-		if ($referenced === NULL || $checkReferenced || $this->checkReferenced) {
+		$referenced = & $this->refCache['referenced'][$this->getSpecificCacheKey()]["$table.$column"];
+		$selection = & $referenced['selection'];
+		$cacheKeys = & $referenced['cacheKeys'];
+		if ($selection === NULL || !isset($cacheKeys[$checkPrimaryKey])) {
 			$this->execute();
-			$this->checkReferenced = FALSE;
-			$keys = array();
+			$cacheKeys = array();
 			foreach ($this->rows as $row) {
 				if ($row[$column] === NULL) {
 					continue;
 				}
 
-				$key = $row[$column] instanceof ActiveRow ? $row[$column]->getPrimary() : $row[$column];
-				$keys[$key] = TRUE;
+				$key = $row[$column];
+				$cacheKeys[$key] = TRUE;
 			}
 
-			if ($referenced !== NULL) {
-				$a = array_keys($keys);
-				$b = array_keys($referenced->rows);
-				sort($a);
-				sort($b);
-				if ($a === $b) {
-					return $referenced;
-				}
-			}
-
-			if ($keys) {
-				$referenced = $this->createSelectionInstance($table);
-				$referenced->where($referenced->getPrimary(), array_keys($keys));
+			if ($cacheKeys) {
+				$selection = $this->createSelectionInstance($table);
+				$selection->where($selection->getPrimary(), array_keys($cacheKeys));
 			} else {
-				$referenced = array();
+				$selection = array();
 			}
 		}
 
-		return $referenced;
+		return $selection;
 	}
 
 
@@ -734,7 +839,7 @@ class Selection extends Nette\Object implements \Iterator, \ArrayAccess, \Counta
 	 */
 	public function getReferencingTable($table, $column, $active = NULL)
 	{
-		$prototype = & $this->getRefTable($refPath)->referencingPrototype[$refPath . "$table.$column"];
+		$prototype = & $this->refCache['referencingPrototype']["$table.$column"];
 		if (!$prototype) {
 			$prototype = $this->createGroupedSelectionInstance($table, $column);
 			$prototype->where("$table.$column", array_keys((array) $this->rows));
@@ -757,7 +862,7 @@ class Selection extends Nette\Object implements \Iterator, \ArrayAccess, \Counta
 	}
 
 
-	/** @return ActiveRow */
+	/** @return IRow */
 	public function current()
 	{
 		if (($key = current($this->keys)) !== FALSE) {
@@ -795,7 +900,7 @@ class Selection extends Nette\Object implements \Iterator, \ArrayAccess, \Counta
 	/**
 	 * Mimic row.
 	 * @param  string row ID
-	 * @param  ActiveRow
+	 * @param  IRow
 	 * @return NULL
 	 */
 	public function offsetSet($key, $value)
@@ -808,7 +913,7 @@ class Selection extends Nette\Object implements \Iterator, \ArrayAccess, \Counta
 	/**
 	 * Returns specified row.
 	 * @param  string row ID
-	 * @return ActiveRow or NULL if there is no such row
+	 * @return IRow or NULL if there is no such row
 	 */
 	public function offsetGet($key)
 	{
