@@ -2,16 +2,13 @@
 
 /**
  * This file is part of the Nette Framework (http://nette.org)
- *
  * Copyright (c) 2004 David Grudl (http://davidgrudl.com)
- *
- * For the full copyright and license information, please view
- * the file license.txt that was distributed with this source code.
  */
 
 namespace Nette\Diagnostics;
 
-use Nette;
+use Nette,
+	ErrorException;
 
 
 /**
@@ -23,20 +20,17 @@ use Nette;
  *
  * @author     David Grudl
  */
-final class Debugger
+class Debugger
 {
 	/** @var bool in production mode is suppressed any debugging output */
-	public static $productionMode;
+	public static $productionMode = self::DETECT;
 
-	/** @var bool in console mode is omitted HTML output */
+	/** @deprecated */
 	public static $consoleMode;
 
 	/** @var int timestamp with microseconds of the start of the request */
 	public static $time;
 
-	/** @var bool is AJAX request detected? */
-	private static $ajaxDetected;
-
 	/** @var string  requested URI or command line */
 	public static $source;
 
@@ -57,19 +51,8 @@ final class Debugger
 	/** @var bool display location? {@link Debugger::dump()} */
 	public static $showLocation = FALSE;
 
-	/** @var array */
-	public static $consoleColors = array(
-		'bool' => '1;33',
-		'null' => '1;33',
-		'int' => '1;36',
-		'float' => '1;36',
-		'string' => '1;32',
-		'array' => '1;31',
-		'key' => '1;37',
-		'object' => '1;31',
-		'visibility' => '1;30',
-		'resource' => '1;37',
-	);
+	/** @deprecated */
+	public static $consoleColors;
 
 	/********************* errors and exceptions reporting ****************d*g**/
 
@@ -78,7 +61,7 @@ final class Debugger
 		PRODUCTION = TRUE,
 		DETECT = NULL;
 
-	/** @var BlueScreen */
+	/** @deprecated @var BlueScreen*/
 	public static $blueScreen;
 
 	/** @var bool|int determines whether any error will cause immediate death; if integer that it's matched against error severity */
@@ -96,38 +79,44 @@ final class Debugger
 	/** @var mixed {@link Debugger::tryError()} FALSE means catching is disabled */
 	private static $lastError = FALSE;
 
+	/** @internal */
+	public static $errorTypes = array(
+		E_ERROR => 'Fatal Error',
+		E_USER_ERROR => 'User Error',
+		E_RECOVERABLE_ERROR => 'Recoverable Error',
+		E_CORE_ERROR => 'Core Error',
+		E_COMPILE_ERROR => 'Compile Error',
+		E_PARSE => 'Parse Error',
+		E_WARNING => 'Warning',
+		E_CORE_WARNING => 'Core Warning',
+		E_COMPILE_WARNING => 'Compile Warning',
+		E_USER_WARNING => 'User Warning',
+		E_NOTICE => 'Notice',
+		E_USER_NOTICE => 'User Notice',
+		E_STRICT => 'Strict standards',
+		E_DEPRECATED => 'Deprecated',
+		E_USER_DEPRECATED => 'User Deprecated',
+	);
+
 	/********************* logging ****************d*g**/
 
-	/** @var Logger */
+	/** @deprecated @var Logger */
 	public static $logger;
 
-	/** @var FireLogger */
+	/** @deprecated @var FireLogger */
 	public static $fireLogger;
 
 	/** @var string name of the directory where errors should be logged; FALSE means that logging is disabled */
 	public static $logDirectory;
 
-	/** @var string email to sent error notifications */
+	/** @var string|array email(s) to which send error notifications */
 	public static $email;
 
 	/** @deprecated */
-	public static $mailer;
+	public static $mailer = array('Nette\Diagnostics\Logger', 'defaultMailer');
 
 	/** @deprecated */
-	public static $emailSnooze;
-
-	/********************* debug bar ****************d*g**/
-
-	/** @var Bar */
-	public static $bar;
-
-	/** @var DefaultBarPanel */
-	private static $errorPanel;
-
-	/** @var DefaultBarPanel */
-	private static $dumpPanel;
-
-	/********************* Firebug extension ****************d*g**/
+	public static $emailSnooze = 172800;
 
 	/** {@link Debugger::log()} and {@link Debugger::fireLog()} */
 	const DEBUG = 'debug',
@@ -136,6 +125,11 @@ final class Debugger
 		ERROR = 'error',
 		CRITICAL = 'critical';
 
+	/********************* debug bar ****************d*g**/
+
+	/** @deprecated @var Bar */
+	public static $bar;
+
 
 	/**
 	 * Static class - cannot be instantiated.
@@ -146,70 +140,6 @@ final class Debugger
 	}
 
 
-	/**
-	 * Static class constructor.
-	 * @internal
-	 */
-	public static function _init()
-	{
-		self::$time = isset($_SERVER['REQUEST_TIME_FLOAT']) ? $_SERVER['REQUEST_TIME_FLOAT'] : microtime(TRUE);
-		self::$consoleMode = PHP_SAPI === 'cli';
-		self::$productionMode = self::DETECT;
-		if (self::$consoleMode) {
-			self::$source = empty($_SERVER['argv']) ? 'cli' : 'cli: ' . implode(' ', $_SERVER['argv']);
-		} else {
-			self::$ajaxDetected = isset($_SERVER['HTTP_X_REQUESTED_WITH']) && $_SERVER['HTTP_X_REQUESTED_WITH'] === 'XMLHttpRequest';
-			if (isset($_SERVER['REQUEST_URI'])) {
-				self::$source = (isset($_SERVER['HTTPS']) && strcasecmp($_SERVER['HTTPS'], 'off') ? 'https://' : 'http://')
-					. (isset($_SERVER['HTTP_HOST']) ? $_SERVER['HTTP_HOST'] : (isset($_SERVER['SERVER_NAME']) ? $_SERVER['SERVER_NAME'] : ''))
-					. $_SERVER['REQUEST_URI'];
-			}
-		}
-
-		self::$logger = new Logger;
-		self::$logDirectory = & self::$logger->directory;
-		self::$email = & self::$logger->email;
-		self::$mailer = & self::$logger->mailer;
-		self::$emailSnooze = & Logger::$emailSnooze;
-
-		self::$fireLogger = new FireLogger;
-
-		self::$blueScreen = new BlueScreen;
-		self::$blueScreen->addPanel(function($e) {
-			if ($e instanceof Nette\Templating\FilterException) {
-				return array(
-					'tab' => 'Template',
-					'panel' => '<p><b>File:</b> ' . Helpers::editorLink($e->sourceFile, $e->sourceLine)
-					. '&nbsp; <b>Line:</b> ' . ($e->sourceLine ? $e->sourceLine : 'n/a') . '</p>'
-					. ($e->sourceLine ? BlueScreen::highlightFile($e->sourceFile, $e->sourceLine) : '')
-				);
-			} elseif ($e instanceof Nette\Utils\NeonException && preg_match('#line (\d+)#', $e->getMessage(), $m)) {
-				if ($item = Helpers::findTrace($e->getTrace(), 'Nette\Config\Adapters\NeonAdapter::load')) {
-					return array(
-						'tab' => 'NEON',
-						'panel' => '<p><b>File:</b> ' . Helpers::editorLink($item['args'][0], $m[1]) . '&nbsp; <b>Line:</b> ' . $m[1] . '</p>'
-							. BlueScreen::highlightFile($item['args'][0], $m[1])
-					);
-				} elseif ($item = Helpers::findTrace($e->getTrace(), 'Nette\Utils\Neon::decode')) {
-					return array(
-						'tab' => 'NEON',
-						'panel' => BlueScreen::highlightPhp($item['args'][0], $m[1])
-					);
-				}
-			}
-		});
-
-		self::$bar = new Bar;
-		self::$bar->addPanel(new DefaultBarPanel('time'));
-		self::$bar->addPanel(new DefaultBarPanel('memory'));
-		self::$bar->addPanel(self::$errorPanel = new DefaultBarPanel('errors')); // filled by _errorHandler()
-		self::$bar->addPanel(self::$dumpPanel = new DefaultBarPanel('dumps')); // filled by barDump()
-	}
-
-
-	/********************* errors and exceptions reporting ****************d*g**/
-
-
 	/**
 	 * Enables displaying or logging errors and exceptions.
 	 * @param  mixed         production, development mode, autodetection or IP address(es) whitelist.
@@ -219,6 +149,15 @@ final class Debugger
 	 */
 	public static function enable($mode = NULL, $logDirectory = NULL, $email = NULL)
 	{
+		self::$time = isset($_SERVER['REQUEST_TIME_FLOAT']) ? $_SERVER['REQUEST_TIME_FLOAT'] : microtime(TRUE);
+		if (isset($_SERVER['REQUEST_URI'])) {
+			self::$source = (isset($_SERVER['HTTPS']) && strcasecmp($_SERVER['HTTPS'], 'off') ? 'https://' : 'http://')
+				. (isset($_SERVER['HTTP_HOST']) ? $_SERVER['HTTP_HOST'] : '')
+				. $_SERVER['REQUEST_URI'];
+		} else {
+			self::$source = empty($_SERVER['argv']) ? 'CLI' : 'CLI: ' . implode(' ', $_SERVER['argv']);
+		}
+		self::$consoleColors = & Dumper::$terminalColors;
 		error_reporting(E_ALL | E_STRICT);
 
 		// production/development mode detection
@@ -241,11 +180,8 @@ final class Debugger
 				echo __METHOD__ . "() error: Log directory is not found or is not directory.\n";
 				exit(254);
 			}
-		} elseif ($logDirectory === FALSE) {
+		} elseif ($logDirectory === FALSE || self::$logDirectory === NULL) {
 			self::$logDirectory = FALSE;
-
-		} elseif (self::$logDirectory === NULL) {
-			self::$logDirectory = defined('APP_DIR') ? APP_DIR . '/../log' : getcwd() . '/log';
 		}
 		if (self::$logDirectory) {
 			ini_set('error_log', self::$logDirectory . '/php_error.log');
@@ -263,30 +199,115 @@ final class Debugger
 		}
 
 		if ($email) {
-			if (!is_string($email)) {
+			if (!is_string($email) && !is_array($email)) {
 				echo __METHOD__ . "() error: Email address must be a string.\n";
 				exit(254);
 			}
 			self::$email = $email;
 		}
 
-		if (!defined('E_DEPRECATED')) {
-			define('E_DEPRECATED', 8192);
-		}
-
-		if (!defined('E_USER_DEPRECATED')) {
-			define('E_USER_DEPRECATED', 16384);
-		}
-
 		if (!self::$enabled) {
 			register_shutdown_function(array(__CLASS__, '_shutdownHandler'));
 			set_exception_handler(array(__CLASS__, '_exceptionHandler'));
 			set_error_handler(array(__CLASS__, '_errorHandler'));
+
+			foreach (array('Nette\Diagnostics\Bar', 'Nette\Diagnostics\BlueScreen', 'Nette\Diagnostics\DefaultBarPanel', 'Nette\Diagnostics\Dumper', 'Nette\Diagnostics\FireLogger',
+				'Nette\Diagnostics\Helpers', 'Nette\Diagnostics\Logger', 'Nette\Utils\Html', 'Nette\Utils\Strings') as $class) {
+				class_exists($class);
+			}
 			self::$enabled = TRUE;
 		}
 	}
 
 
+	/**
+	 * @return BlueScreen
+	 */
+	public static function getBlueScreen()
+	{
+		if (!self::$blueScreen) {
+			self::$blueScreen = new BlueScreen;
+			self::$blueScreen->collapsePaths[] = dirname(__DIR__);
+			self::$blueScreen->addPanel(function($e) {
+				if ($e instanceof Nette\Templating\FilterException) {
+					return array(
+						'tab' => 'Template',
+						'panel' => '<p><b>File:</b> ' . Helpers::editorLink($e->sourceFile, $e->sourceLine) . '</p>'
+						. ($e->sourceLine ? BlueScreen::highlightFile($e->sourceFile, $e->sourceLine) : '')
+					);
+				} elseif ($e instanceof Nette\Utils\NeonException && preg_match('#line (\d+)#', $e->getMessage(), $m)) {
+					if ($item = Helpers::findTrace($e->getTrace(), 'Nette\DI\Config\Adapters\NeonAdapter::load')) {
+						return array(
+							'tab' => 'NEON',
+							'panel' => '<p><b>File:</b> ' . Helpers::editorLink($item['args'][0], $m[1]) . '</p>'
+								. BlueScreen::highlightFile($item['args'][0], $m[1])
+						);
+					} elseif ($item = Helpers::findTrace($e->getTrace(), 'Nette\Utils\Neon::decode')) {
+						return array(
+							'tab' => 'NEON',
+							'panel' => BlueScreen::highlightPhp($item['args'][0], $m[1])
+						);
+					}
+				}
+			});
+		}
+		return self::$blueScreen;
+	}
+
+
+	/**
+	 * @return Bar
+	 */
+	public static function getBar()
+	{
+		if (!self::$bar) {
+			self::$bar = new Bar;
+			self::$bar->addPanel(new DefaultBarPanel('time'));
+			self::$bar->addPanel(new DefaultBarPanel('memory'));
+			self::$bar->addPanel(new DefaultBarPanel('errors'), __CLASS__ . ':errors'); // filled by _errorHandler()
+			self::$bar->addPanel(new DefaultBarPanel('dumps'), __CLASS__ . ':dumps'); // filled by barDump()
+		}
+		return self::$bar;
+	}
+
+
+	/**
+	 * @return void
+	 */
+	public static function setLogger($logger)
+	{
+		self::$logger = $logger;
+	}
+
+
+	/**
+	 * @return Logger
+	 */
+	public static function getLogger()
+	{
+		if (!self::$logger) {
+			self::$logger = new Logger;
+			self::$logger->directory = & self::$logDirectory;
+			self::$logger->email = & self::$email;
+			self::$logger->mailer = & self::$mailer;
+			self::$logger->emailSnooze = & self::$emailSnooze;
+		}
+		return self::$logger;
+	}
+
+
+	/**
+	 * @return FireLogger
+	 */
+	public static function getFireLogger()
+	{
+		if (!self::$fireLogger) {
+			self::$fireLogger = new FireLogger;
+		}
+		return self::$fireLogger;
+	}
+
+
 	/**
 	 * Is Debug enabled?
 	 * @return bool
@@ -312,14 +333,20 @@ final class Debugger
 			throw new Nette\InvalidStateException('Logging directory is not specified in Nette\Diagnostics\Debugger::$logDirectory.');
 		}
 
+		$exceptionFilename = NULL;
 		if ($message instanceof \Exception) {
 			$exception = $message;
-			$message = ($message instanceof Nette\FatalErrorException
-				? 'Fatal error: ' . $exception->getMessage()
-				: get_class($exception) . ": " . $exception->getMessage())
-				. " in " . $exception->getFile() . ":" . $exception->getLine();
+			while ($exception) {
+				$tmp[] = ($exception instanceof ErrorException
+					? 'Fatal error: ' . $exception->getMessage()
+					: get_class($exception) . ": " . $exception->getMessage())
+					. " in " . $exception->getFile() . ":" . $exception->getLine();
+				$exception = $exception->getPrevious();
+			}
+			$exception = $message;
+			$message = implode($tmp, "\ncaused by ");
 
-			$hash = md5(preg_replace('~(Resource id #)\d+~', '$1', $exception ));
+			$hash = md5(preg_replace('~(Resource id #)\d+~', '$1', $exception));
 			$exceptionFilename = "exception-" . @date('Y-m-d-H-i-s') . "-$hash.html";
 			foreach (new \DirectoryIterator(self::$logDirectory) as $entry) {
 				if (strpos($entry, $hash)) {
@@ -328,27 +355,30 @@ final class Debugger
 					break;
 				}
 			}
+		} elseif (!is_string($message)) {
+			$message = Dumper::toText($message);
 		}
 
-		self::$logger->log(array(
-			@date('[Y-m-d H-i-s]'),
-			trim($message),
-			self::$source ? ' @  ' . self::$source : NULL,
-			!empty($exceptionFilename) ? ' @@  ' . $exceptionFilename : NULL
-		), $priority);
-
-		if (!empty($exceptionFilename)) {
+		if ($exceptionFilename) {
 			$exceptionFilename = self::$logDirectory . '/' . $exceptionFilename;
 			if (empty($saved) && $logHandle = @fopen($exceptionFilename, 'w')) {
 				ob_start(); // double buffer prevents sending HTTP headers in some PHP
 				ob_start(function($buffer) use ($logHandle) { fwrite($logHandle, $buffer); }, 4096);
-				self::$blueScreen->render($exception);
+				self::getBlueScreen()->render($exception);
 				ob_end_flush();
 				ob_end_clean();
 				fclose($logHandle);
 			}
-			return strtr($exceptionFilename, '\\/', DIRECTORY_SEPARATOR . DIRECTORY_SEPARATOR);
 		}
+
+		self::getLogger()->log(array(
+			@date('[Y-m-d H-i-s]'),
+			trim($message),
+			self::$source ? ' @  ' . self::$source : NULL,
+			$exceptionFilename ? ' @@  ' . basename($exceptionFilename) : NULL
+		), $priority);
+
+		return $exceptionFilename ? strtr($exceptionFilename, '\\/', DIRECTORY_SEPARATOR . DIRECTORY_SEPARATOR) : NULL;
 	}
 
 
@@ -363,21 +393,12 @@ final class Debugger
 			return;
 		}
 
-		// fatal error handler
-		static $types = array(
-			E_ERROR => 1,
-			E_CORE_ERROR => 1,
-			E_COMPILE_ERROR => 1,
-			E_PARSE => 1,
-		);
 		$error = error_get_last();
-		if (isset($types[$error['type']])) {
-			self::_exceptionHandler(new Nette\FatalErrorException($error['message'], 0, $error['type'], $error['file'], $error['line'], NULL));
-		}
+		if (in_array($error['type'], array(E_ERROR, E_CORE_ERROR, E_COMPILE_ERROR, E_PARSE))) {
+			self::_exceptionHandler(Helpers::fixStack(new ErrorException($error['message'], 0, $error['type'], $error['file'], $error['line'])), TRUE);
 
-		// debug bar (require HTML & development mode)
-		if (self::$bar && !self::$productionMode && self::isHtmlMode()) {
-			self::$bar->render();
+		} elseif (!connection_aborted() && !self::$productionMode && self::isHtmlMode()) {
+			self::getBar()->render();
 		}
 	}
 
@@ -388,9 +409,14 @@ final class Debugger
 	 * @return void
 	 * @internal
 	 */
-	public static function _exceptionHandler(\Exception $exception)
+	public static function _exceptionHandler(\Exception $exception, $shutdown = FALSE)
 	{
-		if (!headers_sent()) { // for PHP < 5.2.4
+		if (!self::$enabled) {
+			return;
+		}
+		self::$enabled = FALSE; // prevent double rendering
+
+		if (!headers_sent()) {
 			$protocol = isset($_SERVER['SERVER_PROTOCOL']) ? $_SERVER['SERVER_PROTOCOL'] : 'HTTP/1.1';
 			$code = isset($_SERVER['HTTP_USER_AGENT']) && strpos($_SERVER['HTTP_USER_AGENT'], 'MSIE ') !== FALSE ? 503 : 500;
 			header("$protocol $code", TRUE, $code);
@@ -404,34 +430,27 @@ final class Debugger
 					echo 'FATAL ERROR: unable to log error';
 				}
 
-				if (self::$consoleMode) {
-					echo "ERROR: the server encountered an internal error and was unable to complete your request.\n";
-
-				} elseif (self::isHtmlMode()) {
+				if (self::isHtmlMode()) {
 					require __DIR__ . '/templates/error.phtml';
+
+				} else {
+					echo "ERROR: the server encountered an internal error and was unable to complete your request.\n";
 				}
 
 			} else {
-				if (self::$consoleMode) { // dump to console
-					echo "$exception\n";
-					if ($file = self::log($exception)) {
-						echo "(stored in $file)\n";
-						if (self::$browser) {
-							exec(self::$browser . ' ' . escapeshellarg($file));
-						}
-					}
+				if (!connection_aborted() && self::isHtmlMode()) {
+					self::getBlueScreen()->render($exception);
+					self::getBar()->render();
 
-				} elseif (self::isHtmlMode()) { // dump to browser
-					self::$blueScreen->render($exception);
-					if (self::$bar) {
-						self::$bar->render();
-					}
-
-				} elseif (!self::fireLog($exception)) { // AJAX or non-HTML mode
+				} elseif (connection_aborted() || !self::fireLog($exception)) {
 					$file = self::log($exception, self::ERROR);
 					if (!headers_sent()) {
 						header("X-Nette-Error-Log: $file");
 					}
+					echo "$exception\n" . ($file ? "(stored in $file)\n" : '');
+					if (self::$browser) {
+						exec(self::$browser . ' ' . escapeshellarg($file));
+					}
 				}
 			}
 
@@ -448,8 +467,9 @@ final class Debugger
 			}
 		}
 
-		self::$enabled = FALSE; // un-register shutdown function
-		exit(254);
+		if (!$shutdown) {
+			exit(254);
+		}
 	}
 
 
@@ -461,7 +481,7 @@ final class Debugger
 	 * @param  int    line number the error was raised at
 	 * @param  array  an array of variables that existed in the scope the error was triggered in
 	 * @return bool   FALSE to call normal error handler, NULL otherwise
-	 * @throws Nette\FatalErrorException
+	 * @throws ErrorException
 	 * @internal
 	 */
 	public static function _errorHandler($severity, $message, $file, $line, $context)
@@ -476,32 +496,28 @@ final class Debugger
 		}
 
 		if ($severity === E_RECOVERABLE_ERROR || $severity === E_USER_ERROR) {
-			if (Helpers::findTrace(debug_backtrace(FALSE), '*::__toString')) {
+			if (Helpers::findTrace(debug_backtrace(PHP_VERSION_ID >= 50306 ? DEBUG_BACKTRACE_IGNORE_ARGS : FALSE), '*::__toString')) {
 				$previous = isset($context['e']) && $context['e'] instanceof \Exception ? $context['e'] : NULL;
-				self::_exceptionHandler(new Nette\FatalErrorException($message, 0, $severity, $file, $line, $context, $previous));
+				$e = new ErrorException($message, 0, $severity, $file, $line, $previous);
+				$e->context = $context;
+				self::_exceptionHandler($e);
 			}
-			throw new Nette\FatalErrorException($message, 0, $severity, $file, $line, $context);
+
+			$e = new ErrorException($message, 0, $severity, $file, $line);
+			$e->context = $context;
+			throw $e;
 
 		} elseif (($severity & error_reporting()) !== $severity) {
 			return FALSE; // calls normal error handler to fill-in error_get_last()
 
 		} elseif (!self::$productionMode && (is_bool(self::$strictMode) ? self::$strictMode : ((self::$strictMode & $severity) === $severity))) {
-			self::_exceptionHandler(new Nette\FatalErrorException($message, 0, $severity, $file, $line, $context));
+			$e = new ErrorException($message, 0, $severity, $file, $line);
+			$e->context = $context;
+			self::_exceptionHandler($e);
 		}
 
-		static $types = array(
-			E_WARNING => 'Warning',
-			E_COMPILE_WARNING => 'Warning', // currently unable to handle
-			E_USER_WARNING => 'Warning',
-			E_NOTICE => 'Notice',
-			E_USER_NOTICE => 'Notice',
-			E_STRICT => 'Strict standards',
-			E_DEPRECATED => 'Deprecated',
-			E_USER_DEPRECATED => 'Deprecated',
-		);
-
-		$message = 'PHP ' . (isset($types[$severity]) ? $types[$severity] : 'Unknown error') . ": $message";
-		$count = & self::$errorPanel->data["$file|$line|$message"];
+		$message = 'PHP ' . (isset(self::$errorTypes[$severity]) ? self::$errorTypes[$severity] : 'Unknown error') . ": $message";
+		$count = & self::getBar()->getPanel(__CLASS__ . ':errors')->data["$file|$line|$message"];
 
 		if ($count++) { // repeated error
 			return NULL;
@@ -511,21 +527,16 @@ final class Debugger
 			return NULL;
 
 		} else {
-			$ok = self::fireLog(new \ErrorException($message, 0, $severity, $file, $line));
-			return !self::isHtmlMode() || (!self::$bar && !$ok) ? FALSE : NULL;
+			self::fireLog(new ErrorException($message, 0, $severity, $file, $line));
+			return self::isHtmlMode() ? NULL : FALSE; // FALSE calls normal error handler
 		}
-
-		return FALSE; // call normal error handler
 	}
 
 
-	/**
-	 * Handles exception thrown in __toString().
-	 * @param  \Exception
-	 * @return void
-	 */
+	/** @deprecated */
 	public static function toStringException(\Exception $exception)
 	{
+		trigger_error(__METHOD__ . '() is deprecated; use trigger_error(..., E_USER_ERROR) instead.', E_USER_DEPRECATED);
 		if (self::$enabled) {
 			self::_exceptionHandler($exception);
 		} else {
@@ -534,12 +545,10 @@ final class Debugger
 	}
 
 
-	/**
-	 * Starts catching potential errors/warnings.
-	 * @return void
-	 */
+	/** @deprecated */
 	public static function tryError()
 	{
+		trigger_error(__METHOD__ . '() is deprecated; use own error handler instead.', E_USER_DEPRECATED);
 		if (!self::$enabled && self::$lastError === FALSE) {
 			set_error_handler(array(__CLASS__, '_errorHandler'));
 		}
@@ -547,13 +556,10 @@ final class Debugger
 	}
 
 
-	/**
-	 * Returns catched error/warning message.
-	 * @param  \ErrorException  catched error
-	 * @return bool
-	 */
+	/** @deprecated */
 	public static function catchError(& $error)
 	{
+		trigger_error(__METHOD__ . '() is deprecated; use own error handler instead.', E_USER_DEPRECATED);
 		if (!self::$enabled && self::$lastError !== FALSE) {
 			restore_error_handler();
 		}
@@ -574,48 +580,23 @@ final class Debugger
 	 */
 	public static function dump($var, $return = FALSE)
 	{
-		if (!$return && self::$productionMode) {
-			return $var;
-		}
-
-		$output = "<pre class=\"nette-dump\">" . Helpers::htmlDump($var) . "</pre>\n";
-
-		if (!$return) {
-			$trace = debug_backtrace(FALSE);
-			$item = Helpers::findTrace($trace, 'dump') ?: Helpers::findTrace($trace, __CLASS__ . '::dump');
-			if (isset($item['file'], $item['line']) && is_file($item['file'])) {
-				$lines = file($item['file']);
-				preg_match('#dump\((.*)\)#', $lines[$item['line'] - 1], $m);
-				$output = substr_replace(
-					$output,
-					' title="' . htmlspecialchars((isset($m[0]) ? "$m[0] \n" : '') . "in file {$item['file']} on line {$item['line']}") . '"',
-					4, 0);
-
-				if (self::$showLocation) {
-					$output = substr_replace(
-						$output,
-						' <small>in ' . Helpers::editorLink($item['file'], $item['line']) . ":{$item['line']}</small>",
-						-8, 0);
-				}
-			}
-		}
-
-		if (self::$consoleMode) {
-			if (self::$consoleColors && substr(getenv('TERM'), 0, 5) === 'xterm') {
-				$output = preg_replace_callback('#<span class="php-(\w+)">|</span>#', function($m) {
-					return "\033[" . (isset($m[1], Debugger::$consoleColors[$m[1]]) ? Debugger::$consoleColors[$m[1]] : '0') . "m";
-				}, $output);
-			}
-			$output = htmlspecialchars_decode(strip_tags($output), ENT_QUOTES);
-		}
-
 		if ($return) {
-			return $output;
+			ob_start();
+			Dumper::dump($var, array(
+				Dumper::DEPTH => self::$maxDepth,
+				Dumper::TRUNCATE => self::$maxLen,
+			));
+			return ob_get_clean();
 
-		} else {
-			echo $output;
-			return $var;
+		} elseif (!self::$productionMode) {
+			Dumper::dump($var, array(
+				Dumper::DEPTH => self::$maxDepth,
+				Dumper::TRUNCATE => self::$maxLen,
+				Dumper::LOCATION => self::$showLocation,
+			));
 		}
+
+		return $var;
 	}
 
 
@@ -638,16 +619,17 @@ final class Debugger
 	 * Dumps information about a variable in Nette Debug Bar.
 	 * @param  mixed  variable to dump
 	 * @param  string optional title
+	 * @param  array  dumper options
 	 * @return mixed  variable itself
 	 */
-	public static function barDump($var, $title = NULL)
+	public static function barDump($var, $title = NULL, array $options = NULL)
 	{
 		if (!self::$productionMode) {
-			$dump = array();
-			foreach ((is_array($var) ? $var : array('' => $var)) as $key => $val) {
-				$dump[$key] = Helpers::clickableDump($val);
-			}
-			self::$dumpPanel->data[] = array('title' => $title, 'dump' => $dump);
+			self::getBar()->getPanel(__CLASS__ . ':dumps')->data[] = array('title' => $title, 'dump' => Dumper::toHtml($var, (array) $options + array(
+				Dumper::DEPTH => self::$maxDepth,
+				Dumper::TRUNCATE => self::$maxLen,
+				Dumper::LOCATION => self::$showLocation,
+			)));
 		}
 		return $var;
 	}
@@ -661,22 +643,22 @@ final class Debugger
 	public static function fireLog($message)
 	{
 		if (!self::$productionMode) {
-			return self::$fireLogger->log($message);
+			return self::getFireLogger()->log($message);
 		}
 	}
 
 
 	private static function isHtmlMode()
 	{
-		return !self::$ajaxDetected && !self::$consoleMode
+		return empty($_SERVER['HTTP_X_REQUESTED_WITH'])
+			&& PHP_SAPI !== 'cli'
 			&& !preg_match('#^Content-Type: (?!text/html)#im', implode("\n", headers_list()));
 	}
 
 
-	/** @deprecated */
 	public static function addPanel(IBarPanel $panel, $id = NULL)
 	{
-		return self::$bar->addPanel($panel, $id);
+		return self::getBar()->addPanel($panel, $id);
 	}
 
 }
