@@ -2,11 +2,7 @@
 
 /**
  * This file is part of the Nette Framework (http://nette.org)
- *
  * Copyright (c) 2004 David Grudl (http://davidgrudl.com)
- *
- * For the full copyright and license information, please view
- * the file license.txt that was distributed with this source code.
  */
 
 namespace Nette\Forms;
@@ -22,12 +18,6 @@ use Nette;
 interface IControl
 {
 
-	/**
-	 * Loads HTTP data.
-	 * @return void
-	 */
-	function loadHttpData();
-
 	/**
 	 * Sets control's value.
 	 * @param  mixed
@@ -42,9 +32,9 @@ interface IControl
 	function getValue();
 
 	/**
-	 * @return Rules
+	 * @return void
 	 */
-	function getRules();
+	function validate();
 
 	/**
 	 * Returns errors corresponding to control.
@@ -53,10 +43,10 @@ interface IControl
 	function getErrors();
 
 	/**
-	 * Is control disabled?
+	 * Is control value excluded from $form->getValues() result?
 	 * @return bool
 	 */
-	function isDisabled();
+	function isOmitted();
 
 	/**
 	 * Returns translated string.
