@@ -2,11 +2,7 @@
 
 /**
  * This file is part of the Nette Framework (http://nette.org)
- *
  * Copyright (c) 2004 David Grudl (http://davidgrudl.com)
- *
- * For the full copyright and license information, please view
- * the file license.txt that was distributed with this source code.
  */
 
 namespace Nette\Forms;
@@ -22,12 +18,6 @@ use Nette;
 interface ISubmitterControl extends IControl
 {
 
-	/**
-	 * Tells if the form was submitted by this button.
-	 * @return bool
-	 */
-	function isSubmittedBy();
-
 	/**
 	 * Gets the validation scope. Clicking the button validates only the controls within the specified scope.
 	 * @return mixed
