@@ -2,11 +2,7 @@
 
 /**
  * This file is part of the Nette Framework (http://nette.org)
- *
  * Copyright (c) 2004 David Grudl (http://davidgrudl.com)
- *
- * For the full copyright and license information, please view
- * the file license.txt that was distributed with this source code.
  */
 
 namespace Nette\Latte;
@@ -32,7 +28,7 @@ class Engine extends Nette\Object
 	{
 		$this->parser = new Parser;
 		$this->compiler = new Compiler;
-		$this->compiler->defaultContentType = Compiler::CONTENT_XHTML;
+		$this->compiler->defaultContentType = Compiler::CONTENT_HTML;
 
 		Macros\CoreMacros::install($this->compiler);
 		$this->compiler->addMacro('cache', new Macros\CacheMacro($this->compiler));
