@@ -2,11 +2,7 @@
 
 /**
  * This file is part of the Nette Framework (http://nette.org)
- *
  * Copyright (c) 2004 David Grudl (http://davidgrudl.com)
- *
- * For the full copyright and license information, please view
- * the file license.txt that was distributed with this source code.
  */
 
 namespace Nette\Latte;
@@ -36,6 +32,9 @@ class HtmlNode extends Nette\Object
 	/** @var bool */
 	public $closing = FALSE;
 
+	/** @var HtmlNode */
+	public $parentNode;
+
 	/** @var string */
 	public $attrCode;
 
@@ -43,9 +42,10 @@ class HtmlNode extends Nette\Object
 	public $offset;
 
 
-	public function __construct($name)
+	public function __construct($name, self $parentNode = NULL)
 	{
 		$this->name = $name;
+		$this->parentNode = $parentNode;
 	}
 
 }
