@@ -2,11 +2,7 @@
 
 /**
  * This file is part of the Nette Framework (http://nette.org)
- *
  * Copyright (c) 2004 David Grudl (http://davidgrudl.com)
- *
- * For the full copyright and license information, please view
- * the file license.txt that was distributed with this source code.
  */
 
 namespace Nette;
@@ -20,6 +16,7 @@ use Nette;
  * @author     David Grudl
  *
  * @property-read bool $frozen
+ * @deprecated
  */
 abstract class FreezableObject extends Object implements IFreezable
 {
@@ -41,7 +38,7 @@ abstract class FreezableObject extends Object implements IFreezable
 	 * Is the object unmodifiable?
 	 * @return bool
 	 */
-	final public function isFrozen()
+	public function isFrozen()
 	{
 		return $this->frozen;
 	}
