@@ -2,11 +2,7 @@
 
 /**
  * This file is part of the Nette Framework (http://nette.org)
- *
  * Copyright (c) 2004 David Grudl (http://davidgrudl.com)
- *
- * For the full copyright and license information, please view
- * the file license.txt that was distributed with this source code.
  */
 
 namespace Nette;
@@ -540,7 +536,7 @@ class Image extends Object
 				return imagepng($this->getImageResource(), $file, $quality);
 
 			case self::GIF:
-				return $file === NULL ? imagegif($this->getImageResource()) : imagegif($this->getImageResource(), $file); // PHP bug #44591
+				return imagegif($this->getImageResource(), $file);
 
 			default:
 				throw new InvalidArgumentException("Unsupported image type.");
